"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.run = exports.processSiteData = exports.renderHtmlFromPage = exports.defaultConfig = void 0;
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const chrome_aws_lambda_1 = __importDefault(require("chrome-aws-lambda"));
const roam_marked_1 = require("roam-marked");
const roam_client_1 = require("roam-client");
const react_1 = __importDefault(require("react"));
const server_1 = __importDefault(require("react-dom/server"));
const jsdom_1 = require("jsdom");
const DailyLog_1 = __importDefault(require("./DailyLog"));
const InlineBlockReference_1 = __importDefault(require("./InlineBlockReference"));
const Header_1 = require("./Header");
const Sidebar_1 = require("./Sidebar");
const ImagePreview_1 = require("./ImagePreview");
const CONFIG_PAGE_NAMES = ["roam/js/static-site", "roam/js/public-garden"];
const IGNORE_BLOCKS = CONFIG_PAGE_NAMES.map((c) => `${c}/ignore`);
const TITLE_REGEX = new RegExp(`(?:${CONFIG_PAGE_NAMES.map((c) => `${c.replace("/", "\\/")}/title`).join("|")})::(.*)`);
const HEAD_REGEX = new RegExp(`(?:${CONFIG_PAGE_NAMES.map((c) => `${c.replace("/", "\\/")}/head`).join("|")})::`);
const DESCRIPTION_REGEX = new RegExp(`(?:${CONFIG_PAGE_NAMES.map((c) => `${c.replace("/", "\\/")}/description`).join("|")})::(.*)`);
const HTML_REGEX = new RegExp("```html\n(.*)```", "s");
const DAILY_NOTE_PAGE_REGEX = /(January|February|March|April|May|June|July|August|September|October|November|December) [0-3]?[0-9](st|nd|rd|th), [0-9][0-9][0-9][0-9]/;
const allBlockMapper = (t) => [
    t,
    ...t.children.flatMap(allBlockMapper),
];
exports.defaultConfig = {
    index: "Website Index",
    filter: [],
    template: `<!DOCTYPE html>
<html>
<head>
<meta charset="utf-8"/>
<meta name="description" content="$\{PAGE_DESCRIPTION}"/>
<meta property="og:description" content="$\{PAGE_DESCRIPTION}">
<title>$\{PAGE_NAME}</title>
<meta property="og:title" content="$\{PAGE_NAME}">
<meta property="og:type" content="website">
</head>
<body>
<div id="content">
$\{PAGE_CONTENT}
</div>
<div id="references">
<ul>
$\{REFERENCES}
</ul>
</div>
</body>
</html>`,
    referenceTemplate: '<li><a href="${LINK}">${REFERENCE}</a></li>',
    plugins: {},
    theme: {},
};
const DEFAULT_STYLE = `<style>
.rm-highlight {
  background-color: hsl(51, 98%, 81%);
  margin: -2px;
  padding: 2px;
}
.rm-bold {
  font-weight: bold;
}
.document-bullet {
  list-style: none;
}
td {
  font-size: 12px;
  min-width: 100px;
  max-height: 20px;
  padding: 8px 16px;
  border: 1px solid grey;
}
table {
  border-spacing: 0;
  border-collapse: collapse;
}
</style>
`;
const renderComponent = ({ Component, id, props, }) => {
    const component = server_1.default.renderToString(react_1.default.createElement("div", { id, className: "roamjs-react-plugin" }, react_1.default.createElement(Component, props)));
    return component;
};
const pageReferences = { current: {} };
const getTitleRuleFromNode = ({ rule: text, values: children }) => {
    const ruleType = text.trim().toUpperCase();
    if (ruleType === "STARTS WITH" && children.length) {
        const tag = roam_client_1.extractTag(children[0]);
        return (title) => {
            return title.startsWith(tag);
        };
    }
    else if (ruleType === "DAILY") {
        return (title) => roam_client_1.DAILY_NOTE_PAGE_TITLE_REGEX.test(title);
    }
    else if (ruleType === "ALL") {
        return () => true;
    }
    else if (ruleType === "TAGGED WITH" && children.length) {
        const tag = roam_client_1.extractTag(children[0]);
        const references = pageReferences.current[tag] || [];
        return (title) => references.includes(title);
    }
    return undefined;
};
const getParsedTree = ({ page, pageName, }) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        return yield page.evaluate((pageName) => window.getTreeByPageName(pageName), pageName);
    }
    catch (e) {
        console.error(`Failed to get Tree for ${pageName}`);
        throw new Error(e);
    }
});
const getConfigFromPage = (parsedTree) => {
    var _a, _b, _c, _d;
    const getConfigNode = (key) => parsedTree.find((n) => n.text.trim().toUpperCase() === key.toUpperCase());
    const indexNode = getConfigNode("index");
    const filterNode = getConfigNode("filter");
    const templateNode = getConfigNode("template");
    const referenceTemplateNode = getConfigNode("reference template");
    const pluginsNode = getConfigNode("plugins");
    const themeNode = getConfigNode("theme");
    const getCode = (node) => { var _a; return (_a = ((node === null || node === void 0 ? void 0 : node.children) || [])
        .map((s) => s.text.match(HTML_REGEX))
        .find((s) => !!s)) === null || _a === void 0 ? void 0 : _a[1]; };
    const template = getCode(templateNode);
    const referenceTemplate = getCode(referenceTemplateNode);
    const withIndex = ((_a = indexNode === null || indexNode === void 0 ? void 0 : indexNode.children) === null || _a === void 0 ? void 0 : _a.length) ? { index: roam_client_1.extractTag(indexNode.children[0].text.trim()) }
        : {};
    const withFilter = ((_b = filterNode === null || filterNode === void 0 ? void 0 : filterNode.children) === null || _b === void 0 ? void 0 : _b.length) ? {
        filter: filterNode.children.map((t) => ({
            rule: t.text,
            values: t.children.map((c) => c.text),
        })),
    }
        : {};
    const withTemplate = template
        ? {
            template,
        }
        : {};
    const withReferenceTemplate = referenceTemplate
        ? { referenceTemplate }
        : {};
    const withPlugins = ((_c = pluginsNode === null || pluginsNode === void 0 ? void 0 : pluginsNode.children) === null || _c === void 0 ? void 0 : _c.length) ? {
        plugins: Object.fromEntries(pluginsNode.children.map((p) => [
            p.text,
            Object.fromEntries((p.children || []).map((c) => [
                c.text,
                c.children.map((v) => v.text),
            ])),
        ])),
    }
        : {};
    const withTheme = ((_d = themeNode === null || themeNode === void 0 ? void 0 : themeNode.children) === null || _d === void 0 ? void 0 : _d.length) ? {
        theme: Object.fromEntries(themeNode.children.map((p) => [
            p.text,
            Object.fromEntries((p.children || []).map((c) => { var _a; return [c.text, (_a = c.children[0]) === null || _a === void 0 ? void 0 : _a.text]; })),
        ])),
    }
        : {};
    return Object.assign(Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({}, withIndex), withFilter), withTemplate), withReferenceTemplate), withPlugins), withTheme);
};
const prepareContent = ({ content }) => {
    const filterIgnore = (t) => {
        if (IGNORE_BLOCKS.some((ib) => t.text.trim().includes(ib))) {
            return false;
        }
        t.children = t.children.filter(filterIgnore);
        return true;
    };
    return content.filter(filterIgnore);
};
const VIEW_CONTAINER = {
    bullet: "ul",
    document: "div",
    numbered: "ol",
};
const HEADINGS = ["p", "h1", "h2", "h3"];
const convertContentToHtml = ({ content, viewType, level, context, useInlineBlockReferences, pageNameSet, }) => {
    if (content.length === 0) {
        return "";
    }
    const items = content.map((t) => {
        let skipChildren = false;
        const componentsWithChildren = (s, ac) => {
            var _a, _b;
            const parent = context.components(s, ac);
            if (parent) {
                return parent;
            }
            if (/table/i.test(s)) {
                skipChildren = true;
                return `<table><tbody>${t.children
                    .map((row) => `<tr>${[row, ...row.children.flatMap(allBlockMapper)]
                    .map((td) => `<td>${roam_marked_1.parseInline(td.text, Object.assign(Object.assign({}, context), { components: componentsWithChildren }))}</td>`)
                    .join("")}</tr>`)
                    .join("")}</tbody></table>`;
            }
            else if (/static site/i.test(s) && ac) {
                if (/inject/i.test(ac)) {
                    const node = (_a = t.children.find((c) => HTML_REGEX.test(c.text))) === null || _a === void 0 ? void 0 : _a.text;
                    if (node) {
                        skipChildren = true;
                        return ((_b = node.match(HTML_REGEX)) === null || _b === void 0 ? void 0 : _b[1]) || false;
                    }
                }
            }
            return false;
        };
        const classlist = ["roam-block"];
        const textToParse = t.text.replace(/#\.([^\s]*)/g, (_, className) => {
            classlist.push(className);
            return "";
        });
        const inlineMarked = roam_marked_1.parseInline(textToParse, Object.assign(Object.assign({}, context), { components: componentsWithChildren }));
        const children = skipChildren
            ? ""
            : convertContentToHtml({
                content: t.children,
                viewType: t.viewType,
                useInlineBlockReferences,
                level: level + 1,
                context,
                pageNameSet,
            });
        const innerHtml = `<${HEADINGS[t.heading]}>${inlineMarked}</${HEADINGS[t.heading]}>${useInlineBlockReferences
            ? renderComponent({
                Component: InlineBlockReference_1.default,
                id: `${t.uid}-inline-references`,
                props: {
                    blockReferences: t.references.filter((tr) => pageNameSet.has(tr.title)),
                },
            })
            : ""}\n${children}`;
        if (level > 0 && viewType === "document") {
            classlist.push("document-bullet");
        }
        const attrs = `id="${t.uid}" class="${classlist.join(" ")}"`;
        const blockHtml = level === 0 && viewType === "document"
            ? `<div ${attrs}>${innerHtml}</div>`
            : `<li ${attrs}>${innerHtml}</li>`;
        return blockHtml;
    });
    const containerTag = level > 0 && viewType === "document" ? "ul" : VIEW_CONTAINER[viewType];
    return `<${containerTag}>${items.join("\n")}</${containerTag}>`;
};
const PLUGIN_RENDER = {
    header: Header_1.render,
    sidebar: Sidebar_1.render,
    "image-preview": ImagePreview_1.render,
};
const renderHtmlFromPage = ({ outputPath, pageContent, p, config, pageMetadata, blockReferences, theme, }) => {
    const { content, references = [], title, head, description } = pageContent;
    const pageNameSet = new Set(Object.keys(pageMetadata));
    const preparedContent = prepareContent({
        content,
    });
    const useUid = !!config.plugins["uid-paths"];
    const convertPageNameToPath = (name) => name === config.index
        ? "/"
        : useUid
            ? pageMetadata[name]
            : `${encodeURIComponent(name.replace(/ /g, "_").replace(/[",?#:$;/@&=+']/g, ""))}`;
    const htmlFileName = convertPageNameToPath(p);
    const pagesToHrefs = (name) => pageNameSet.has(name)
        ? `/${convertPageNameToPath(name).replace(/^\/$/, "")}`
        : "";
    const pluginKeys = Object.keys(config.plugins);
    const useInlineBlockReferences = pluginKeys.includes("inline-block-references");
    const markedContent = convertContentToHtml({
        content: preparedContent,
        viewType: pageContent.viewType,
        useInlineBlockReferences,
        pageNameSet,
        level: 0,
        context: {
            pagesToHrefs,
            components: (s, ac) => {
                var _a, _b, _c;
                if (/static site/i.test(s)) {
                    if (ac && /daily log/i.test(ac)) {
                        const referenceContent = references
                            .filter(({ title }) => DAILY_NOTE_PAGE_REGEX.test(title))
                            .sort(({ title: a }, { title: b }) => roam_client_1.parseRoamDate(b).valueOf() - roam_client_1.parseRoamDate(a).valueOf())
                            .map(({ node, title }) => (Object.assign(Object.assign({}, node), { text: node.text.replace(p, title) })));
                        const preparedReferenceContent = prepareContent({
                            content: referenceContent,
                        });
                        const firstNode = preparedReferenceContent[0];
                        const firstDate = roam_client_1.parseRoamDate(((_c = (_b = (_a = firstNode === null || firstNode === void 0 ? void 0 : firstNode.text) === null || _a === void 0 ? void 0 : _a.match) === null || _b === void 0 ? void 0 : _b.call(_a, DAILY_NOTE_PAGE_REGEX)) === null || _c === void 0 ? void 0 : _c[0]) || "");
                        const allContent = preparedReferenceContent.slice(1).reduce((prev, cur) => {
                            var _a;
                            const lastNode = prev[prev.length - 1];
                            const curDate = roam_client_1.parseRoamDate(((_a = cur.text.match(DAILY_NOTE_PAGE_REGEX)) === null || _a === void 0 ? void 0 : _a[0]) || "");
                            if (lastNode.month === curDate.getMonth() &&
                                lastNode.year === curDate.getFullYear()) {
                                lastNode.nodes.push(cur);
                                return prev;
                            }
                            else {
                                return [
                                    ...prev,
                                    {
                                        nodes: [cur],
                                        month: curDate.getMonth(),
                                        year: curDate.getFullYear(),
                                    },
                                ];
                            }
                        }, firstNode
                            ? [
                                {
                                    nodes: [firstNode],
                                    month: firstDate.getMonth(),
                                    year: firstDate.getFullYear(),
                                },
                            ]
                            : []);
                        return `${renderComponent({
                            Component: DailyLog_1.default,
                            id: `${p}-daily-log`,
                            props: {
                                allContent: allContent.map((_a) => {
                                    var { nodes } = _a, p = __rest(_a, ["nodes"]);
                                    return (Object.assign(Object.assign({}, p), { html: convertContentToHtml({
                                            content: nodes,
                                            viewType: pageContent.viewType,
                                            useInlineBlockReferences,
                                            pageNameSet,
                                            level: 0,
                                            context: {
                                                pagesToHrefs,
                                                components: () => "",
                                                blockReferences,
                                            },
                                        }) }));
                                }),
                            },
                        })}`;
                    }
                }
                return "";
            },
            blockReferences,
        },
    });
    const hydratedHtml = config.template
        .replace("</head>", `${DEFAULT_STYLE.replace(/<\/style>/, theme)}${head}</head>`)
        .replace(/\${PAGE_NAME}/g, title)
        .replace(/\${PAGE_DESCRIPTION}/g, description)
        .replace(/\${PAGE_CONTENT}/g, markedContent)
        .replace(/\${REFERENCES}/g, Array.from(new Set(references.map((r) => r.title)))
        .filter((r) => pageNameSet.has(r))
        .map((r) => config.referenceTemplate
        .replace(/\${REFERENCE}/g, r)
        .replace(/\${LINK}/g, convertPageNameToPath(r)))
        .join("\n"));
    const dom = new jsdom_1.JSDOM(hydratedHtml);
    pluginKeys.forEach((k) => { var _a; return (_a = PLUGIN_RENDER[k]) === null || _a === void 0 ? void 0 : _a.call(PLUGIN_RENDER, dom, config.plugins[k], {
        convertPageNameToPath,
        references,
        pageName: p,
    }); });
    const newHtml = dom.serialize();
    const fileName = htmlFileName === "/" ? "index.html" : `${htmlFileName}.html`;
    fs_1.default.writeFileSync(path_1.default.join(outputPath, fileName), newHtml);
};
exports.renderHtmlFromPage = renderHtmlFromPage;
const processSiteData = ({ pages, outputPath, config, info, }) => {
    const pageNames = Object.keys(pages).sort();
    info(`resolving ${pageNames.length} pages ${new Date().toLocaleTimeString()}`);
    info(`Here are some: ${pageNames.slice(0, 5)}`);
    const blockReferencesCache = {};
    pageNames.forEach((page) => {
        const { content } = pages[page];
        const forEach = (t) => {
            blockReferencesCache[t.uid] = { text: t.text, page };
            t.children.forEach(forEach);
        };
        content.forEach(forEach);
    });
    const pageMetadata = Object.fromEntries(pageNames.map((p) => [p, pages[p].uid]));
    let theme = "</style>\n";
    if (config.theme.text) {
        if (config.theme.text.font) {
            theme = `body {\n  font-family: ${config.theme.text.font};\n}\n${theme}`;
        }
    }
    if (config.theme.layout) {
        const { width } = config.theme.layout;
        if (width) {
            const widthStyle = /\d$/.test(width) ? `${width}px` : width;
            theme = `#content, #references {\n  margin: auto;\n  width: ${widthStyle};\n}\n${theme}`;
        }
    }
    pageNames.map((p) => {
        exports.renderHtmlFromPage({
            outputPath,
            config,
            pageContent: pages[p],
            p,
            pageMetadata,
            blockReferences: (t) => blockReferencesCache[t],
            theme,
        });
    });
    return config;
};
exports.processSiteData = processSiteData;
const run = ({ roamUsername, roamPassword, roamGraph, logger = { info: console.log, error: console.error }, pathRoot = process.cwd(), inputConfig = {}, debug = false, }) => __awaiter(void 0, void 0, void 0, function* () {
    const { info, error } = logger;
    info(`Hello ${roamUsername}! Fetching from ${roamGraph}... ${new Date().toLocaleTimeString()}`);
    const chromiumPath = yield chrome_aws_lambda_1.default.executablePath;
    const executablePath = chromiumPath
        ? chromiumPath
        : process.platform === "win32"
            ? "C:\\Program Files (x86)\\Google\\Chrome\\Application\\chrome.exe"
            : "/usr/bin/google-chrome-stable";
    return chrome_aws_lambda_1.default.puppeteer
        .launch({
        args: chrome_aws_lambda_1.default.args,
        defaultViewport: chrome_aws_lambda_1.default.defaultViewport,
        executablePath,
        headless: true,
        ignoreHTTPSErrors: true,
    })
        .then((browser) => __awaiter(void 0, void 0, void 0, function* () {
        const page = yield browser.newPage();
        try {
            const outputPath = path_1.default.join(pathRoot, "out");
            fs_1.default.mkdirSync(outputPath, { recursive: true });
            yield page.goto("https://roamresearch.com/#/signin?disablejs=true", {
                waitUntil: "networkidle0",
            });
            // Roam's doing this weird refresh thing. let's just hardcode it
            yield page.waitForTimeout(5000);
            yield page.waitForSelector("input[name=email]", {
                timeout: 120000,
            });
            yield page.type("input[name=email]", roamUsername);
            yield page.type("input[name=password]", roamPassword);
            yield page.click("button.bp3-button");
            info(`Signing in ${new Date().toLocaleTimeString()}`);
            yield page.waitForSelector(`a[href="#/app/${roamGraph}"]`, {
                timeout: 120000,
            });
            info(`Done waiting for graph to be selectable ${new Date().toLocaleTimeString()}`);
            yield page.evaluate((roamGraph) => { var _a; return (_a = document
                .querySelector(`a[href="#/app/${roamGraph}"]`)) === null || _a === void 0 ? void 0 : _a.scrollIntoView(); }, roamGraph);
            yield page.waitForTimeout(5000);
            info(`Done waiting for page to scroll ${new Date().toLocaleTimeString()}`);
            yield page.click(`a[href="#/app/${roamGraph}"]`);
            info(`entering graph ${new Date().toLocaleTimeString()}`);
            yield page.waitForSelector("span.bp3-icon-more", {
                timeout: 120000,
            });
            const allPageNames = yield page.evaluate(() => {
                return window.roamAlphaAPI
                    .q("[:find ?s :where [?e :node/title ?s]]")
                    .map((b) => b[0]);
            });
            yield page.evaluate(() => {
                window.getTreeByBlockId = (blockId) => {
                    var _a;
                    const block = window.roamAlphaAPI.pull("[:block/children, :block/string, :block/order, :block/uid, :block/heading, :block/open, :children/view-type]", blockId);
                    const children = block[":block/children"] || [];
                    const uid = block[":block/uid"] || "";
                    const props = block[":block/props"] || {};
                    return {
                        text: block[":block/string"] || "",
                        order: block[":block/order"] || 0,
                        uid,
                        children: children
                            .map((c) => window.getTreeByBlockId(c[":db/id"]))
                            .sort((a, b) => a.order - b.order),
                        heading: block[":block/heading"] || 0,
                        open: block[":block/open"] || true,
                        viewType: (_a = block[":children/view-type"]) === null || _a === void 0 ? void 0 : _a.substring(1),
                        editTime: new Date(block[":edit/time"] || 0),
                        textAlign: block[":block/text-align"] || "left",
                        props: {
                            imageResize: Object.fromEntries(Object.keys(props[":image-size"] || {}).map((p) => [
                                p,
                                {
                                    height: props[":image-size"][p][":height"],
                                    width: props[":image-size"][p][":width"],
                                },
                            ])),
                            iframe: Object.fromEntries(Object.keys(props[":iframe"] || {}).map((p) => [
                                p,
                                {
                                    height: props[":iframe"][p][":size"][":height"],
                                    width: props[":iframe"][p][":size"][":width"],
                                },
                            ])),
                        },
                    };
                };
                window.fixViewType = ({ c, v, }) => {
                    if (!c.viewType) {
                        c.viewType = v;
                    }
                    c.children.forEach((cc) => window.fixViewType({ c: cc, v: c.viewType }));
                    return c;
                };
                window.getTreeByPageName = (name) => {
                    const result = window.roamAlphaAPI.q(`[:find (pull ?e [:block/children :children/view-type]) :where [?e :node/title "${name
                        .replace(/\\/, "\\\\")
                        .replace(/"/g, '\\"')}"]]`);
                    if (!result.length) {
                        return [];
                    }
                    const block = result[0][0];
                    const children = (block === null || block === void 0 ? void 0 : block.children) || [];
                    const viewType = (block === null || block === void 0 ? void 0 : block["view-type"]) || "bullet";
                    return children
                        .map((c) => window.getTreeByBlockId(c.id))
                        .sort((a, b) => a.order - b.order)
                        .map((c) => window.fixViewType({ c, v: viewType }));
                };
            });
            const configPage = allPageNames.find((c) => CONFIG_PAGE_NAMES.includes(c)) || "";
            const configPageTree = configPage
                ? yield getParsedTree({ page, pageName: configPage })
                : [];
            const userConfig = getConfigFromPage(configPageTree);
            const config = Object.assign(Object.assign(Object.assign({}, exports.defaultConfig), userConfig), inputConfig);
            const blockReferences = config.plugins["inline-block-references"]
                ? yield page.evaluate(() => {
                    return window.roamAlphaAPI
                        .q("[:find ?pu ?pt ?ru :where [?pp :node/title ?pt] [?p :block/page ?pp] [?p :block/uid ?pu] [?r :block/uid ?ru] [?p :block/refs ?r]]")
                        .reduce((cur, [uid, title, u]) => {
                        if (cur[u]) {
                            cur[u].push({ uid, title });
                        }
                        else {
                            cur[u] = [{ uid, title }];
                        }
                        return cur;
                    }, {});
                })
                : {};
            pageReferences.current = yield page
                .evaluate(() => window.roamAlphaAPI.q("[:find ?t ?title :where [?parent :node/title ?title] [?b :block/page ?parent] [?b :block/refs ?p] [?p :node/title ?t]]"))
                .then((prs) => prs.reduce((prev, cur) => (Object.assign(Object.assign({}, prev), { [cur[0]]: [...(prev[cur[0]] || []), cur[1]] })), {}));
            const getReferences = (t) => (Object.assign(Object.assign({}, t), { references: blockReferences[t.uid] || [], children: t.children.map(getReferences) }));
            const titleFilters = config.filter
                .map(getTitleRuleFromNode)
                .filter((f) => !!f);
            const titleFilter = (t) => !titleFilters.length || titleFilters.some((r) => r && r(t));
            info(`querying data ${new Date().toLocaleTimeString()}`);
            const pageNamesWithContent = yield Promise.all(allPageNames
                .filter((pageName) => pageName === config.index || titleFilter(pageName))
                .filter((pageName) => !CONFIG_PAGE_NAMES.includes(pageName))
                .map((pageName) => {
                if (debug) {
                    info(`Getting parsed tree for page ${pageName}`);
                }
                return getParsedTree({ page, pageName }).then((content) => ({
                    pageName,
                    content,
                }));
            }));
            info(`title filtered to ${pageNamesWithContent.length} pages ${new Date().toLocaleTimeString()}`);
            const entries = yield Promise.all(pageNamesWithContent.map(({ pageName, content }) => {
                return Promise.all([
                    page.evaluate((pageName) => window.roamAlphaAPI
                        .q(`[:find ?rt ?r :where [?pr :node/title ?rt] [?r :block/page ?pr] [?r :block/refs ?p] [?p :node/title "${pageName
                        .replace(/\\/, "\\\\")
                        .replace(/"/g, '\\"')}"]]`)
                        .map((args) => ({
                        title: args[0],
                        node: window.fixViewType({
                            c: window.getTreeByBlockId(args[1]),
                            v: "bullet",
                        }),
                    })), pageName),
                    page.evaluate((pageName) => {
                        var _a, _b;
                        return ((_b = (_a = window.roamAlphaAPI.q(`[:find ?v :where [?e :children/view-type ?v] [?e :node/title "${pageName
                            .replace(/\\/, "\\\\")
                            .replace(/"/g, '\\"')}"]]`)) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b[0]) || "bullet";
                    }, pageName),
                    page.evaluate((pageName) => {
                        var _a, _b;
                        return ((_b = (_a = window.roamAlphaAPI.q(`[:find ?u :where [?e :block/uid ?u] [?e :node/title "${pageName
                            .replace(/\\/, "\\\\")
                            .replace(/"/g, '\\"')}"]]`)) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b[0]) || "bullet";
                    }, pageName),
                ])
                    .then(([references, viewType, uid]) => ({
                    references: references.map((r) => (Object.assign(Object.assign({}, r), { node: getReferences(r.node) }))),
                    pageName,
                    content: content.map(getReferences),
                    viewType,
                    uid,
                }))
                    .catch((e) => {
                    console.error("Failed to find references for page", pageName);
                    throw new Error(e);
                });
            }));
            info(`content filtered to ${entries.length} entries ${new Date().toLocaleTimeString()}`);
            const pages = Object.fromEntries(entries.map((_a) => {
                var _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m;
                var { content, pageName } = _a, props = __rest(_a, ["content", "pageName"]);
                const allBlocks = content.flatMap(allBlockMapper);
                const titleMatch = (_d = (_c = (_b = allBlocks
                    .find((s) => TITLE_REGEX.test(s.text))) === null || _b === void 0 ? void 0 : _b.text) === null || _c === void 0 ? void 0 : _c.match) === null || _d === void 0 ? void 0 : _d.call(_c, TITLE_REGEX);
                const headMatch = (_j = (_h = (_g = (_f = (_e = allBlocks
                    .find((s) => HEAD_REGEX.test(s.text))) === null || _e === void 0 ? void 0 : _e.children) === null || _f === void 0 ? void 0 : _f[0]) === null || _g === void 0 ? void 0 : _g.text) === null || _h === void 0 ? void 0 : _h.match) === null || _j === void 0 ? void 0 : _j.call(_h, HTML_REGEX);
                const descriptionMatch = (_m = (_l = (_k = allBlocks
                    .find((s) => DESCRIPTION_REGEX.test(s.text))) === null || _k === void 0 ? void 0 : _k.text) === null || _l === void 0 ? void 0 : _l.match) === null || _m === void 0 ? void 0 : _m.call(_l, DESCRIPTION_REGEX);
                const title = titleMatch ? titleMatch[1].trim() : pageName;
                const head = headMatch ? headMatch[1] : "";
                const description = descriptionMatch
                    ? descriptionMatch[1].trim()
                    : "";
                return [
                    pageName,
                    Object.assign({ content,
                        title,
                        head,
                        description }, props),
                ];
            }));
            yield page.close();
            browser.close();
            return { pages, outputPath, config };
        }
        catch (e) {
            yield page.screenshot({ path: path_1.default.join(pathRoot, "error.png") });
            error("took screenshot");
            throw new Error(e);
        }
    }))
        .then((d) => exports.processSiteData(Object.assign(Object.assign({}, d), { info })))
        .catch((e) => {
        error(e.message);
        throw new Error(e);
    });
});
exports.run = run;
exports.default = exports.run;
